@@ -3,6 +3,7 @@ from tempfile import TemporaryDirectory
 
 from gftools.builder.file import File
 from gftools.builder.operations import OperationBase
+from glyphsLib.builder.axes import find_base_style
 
 
 class Glyphs2DS(OperationBase):
@@ -18,9 +19,14 @@ class Glyphs2DS(OperationBase):
         if "directory" in self.original:
             target = self.original["directory"]
 
-        dspath = os.path.join(
-            target, self.first_source.basename.rsplit(".", 1)[0] + ".designspace"
-        )
+        base_family = self.first_source.family_name
+        # glyphsLib has a special case for Glyphs files where the masters have a
+        # common style name; typically this manifests itself in single-master
+        # Glyphs files, but it can also happen in multi-master files.
+        # In that case, the designspace filename will be changed.
+        if base_style := find_base_style(self.first_source.gsfont.masters):
+            base_family += "-" + base_style
+        dspath = os.path.join(target, base_family.replace(" ", "") + ".designspace")
         return [File(dspath)]
 
     @property
