@@ -1,3 +1,4 @@
+from tempfile import TemporaryDirectory
 import copy
 import logging
 import os
@@ -110,6 +111,17 @@ class GFBuilder(RecipeProviderBase):
         else:
             self.avar2file = None
 
+        if "fvarInstances" in self.config:
+            self.fvarInstancesFile = NamedTemporaryFile(delete=False, mode="w+")
+            for font in list(self.config["fvarInstances"].keys()):
+                scfont = re.sub(r"((?:-Italic)?\[)", r"SC\1", font)
+                self.config["fvarInstances"][scfont] = self.config["fvarInstances"][
+                    font
+                ]
+            yaml.dump(self.config["fvarInstances"], self.fvarInstancesFile)
+        else:
+            self.fvarInstancesFile = None
+
         # Find variable fonts
         self.recipe = {}
         self.build_all_variables()
@@ -259,6 +271,8 @@ class GFBuilder(RecipeProviderBase):
         self.build_STAT()
         if "avar2" in self.config:
             self.build_avar2()
+        if "fvarInstances" in self.config:
+            self.build_fvar_instances()
 
     def build_STAT(self):
         # Add buildStat to a variable target, it'll do for all of them
@@ -290,6 +304,17 @@ class GFBuilder(RecipeProviderBase):
                 self.recipe[vf].append(args)
             self.avar2file.close()
 
+    def build_fvar_instances(self):
+        vfs = [x for x in self.recipe.keys() if x.endswith("ttf")]
+        if len(vfs) > 0:
+            args = {
+                "args": self.fvarInstancesFile.name,
+                "postprocess": "buildFvarInstances",
+            }
+            for vf in vfs:
+                self.recipe[vf].append(args)
+            self.fvarInstancesFile.close()
+
     def _vtt_steps(self, target: str):
         if os.path.basename(target) in self.config.get("vttSources", {}):
             return [
@@ -313,8 +338,9 @@ class GFBuilder(RecipeProviderBase):
                 source, suffix=suffix, italic_ds=italic_ds, roman=False
             )
         steps = (
-            [
-                {"source": source.path},
+            [{"source": source.path}]
+            + self._subset_steps(source, None)
+            + [
                 {
                     "operation": "buildVariable",
                     "args": self.fontmake_args(source, variable=True),
@@ -343,6 +369,42 @@ class GFBuilder(RecipeProviderBase):
                 self._vf_filename(source, italic_ds=italic_ds, roman=roman, suffix="SC")
             ] = self._smallcap_steps(source, target)
 
+    def _subset_steps(
+        self, source: File, instance: Optional[InstanceDescriptor] = None
+    ):
+        if not self.config.get("includeSubsets"):
+            return []
+        if not hasattr(self, "_subset_tempdir"):
+            self._subset_tempdir = TemporaryDirectory()  # Stow object
+        steps = []
+        if source.is_glyphs:
+            steps.append(
+                {
+                    "operation": "glyphs2ds",  # GlyphData?
+                }
+            )
+        steps += [
+            {
+                "operation": "addSubset",
+                "subsets": self.config["includeSubsets"],
+                "directory": self._subset_tempdir.name,
+                "args": "--allow-sparse",
+            }
+        ]
+        if instance:
+            steps += [
+                {
+                    "operation": "instantiateUfo",
+                    "instance_name": instance.name,
+                    "glyphData": self.config.get("glyphData"),
+                    "target": os.path.join(
+                        self._subset_tempdir.name,
+                        instance.filename + ".json",
+                    ),
+                }
+            ]
+        return steps
+
     def build_all_statics(self):
         if not self.config.get("buildStatic", True):
             return
@@ -359,9 +421,17 @@ class GFBuilder(RecipeProviderBase):
 
         steps = [
             {"source": source.path},
-        ]
-        # if we're running fontc we skip conversion to UFO
-        if not source.is_ufo and not self.config.get("use_fontc", False):
+        ] + self._subset_steps(
+            source,
+            instance,
+        )
+        # We skip conversion to UFO if we're running fontc
+        use_fontc = self.config.get("use_fontc", False)
+        if (
+            not source.is_ufo
+            and not self.config.get("includeSubsets")
+            and not use_fontc
+        ):
             instancename = instance.name
             if instancename is None:
                 if not instance.familyName or not instance.styleName:
