@@ -17,6 +17,8 @@ from strictyaml import (
     MapCombined,
 )
 
+from gftools.subsetmerger import subsets_schema
+
 CATEGORIES = ["DISPLAY", "SERIF", "SANS_SERIF", "HANDWRITING", "MONOSPACE"]
 
 
@@ -65,6 +67,18 @@ avar2_schema = MapPattern(
     ),
 )
 
+fvar_instances_schema = MapPattern(
+    Str(),
+    Seq(
+        Map(
+            {
+                "name": Str(),
+                "coordinates": MapPattern(Str(), Float() | Int()),
+            }
+        )
+    ),
+)
+
 stat_format4_schema = Seq(
     Map(
         {
@@ -89,6 +103,7 @@ GOOGLEFONTS_SCHEMA = Map(
         Optional("statFormat4"): stat_format4_schema
         | MapPattern(Str(), stat_format4_schema),
         Optional("avar2"): avar2_schema,
+        Optional("fvarInstances"): fvar_instances_schema,
         Optional("familyName"): Str(),
         Optional("includeSourceFixes"): Bool(),
         Optional("stylespaceFile"): Str(),
@@ -125,5 +140,6 @@ GOOGLEFONTS_SCHEMA = Map(
         Optional("buildSmallCap"): Bool(),
         Optional("splitItalic"): Bool(),
         Optional("localMetadata"): Any(),
+        Optional("includeSubsets"): subsets_schema,
     }
 )
