@@ -16,6 +16,7 @@
 from __future__ import annotations
 from typing import Union
 import requests
+from urllib.parse import urljoin
 from io import BytesIO
 from zipfile import ZipFile
 import sys
@@ -27,7 +28,7 @@ from unidecode import unidecode
 from collections import namedtuple
 from gflanguages import LoadLanguages
 from github import Github
-from pkg_resources import resource_filename
+import importlib.resources
 from google.protobuf import text_format
 import json
 from PIL import Image
@@ -64,7 +65,7 @@ def download_family_from_Google_Fonts(
     res = []
     for item in data["manifest"]["fileRefs"]:
         filename = item["filename"]
-        dl_url = item["url"]
+        dl_url = urljoin(dl_url, item["url"])
         if "static" in filename and ignore_static:
             continue
         if not filename.endswith(("otf", "ttf")):
@@ -520,10 +521,12 @@ def has_mac_names(ttfont):
     """Check if a font has Mac names. Mac names have the following
     field values:
     platformID: 1, encodingID: 0, LanguageID: 0"""
-    for i in range(255):
-        if ttfont["name"].getName(i, 1, 0, 0):
-            return True
-    return False
+    return any(
+        namerecord.platformID == 1
+        and namerecord.platEncID == 0
+        and namerecord.langID == 0
+        for namerecord in ttfont["name"].names
+    )
 
 
 def font_is_italic(ttfont):
@@ -537,8 +540,9 @@ def font_sample_text(ttFont):
     that can be formed using the ttFont instance.
 
     UDHR has been chosen due to the many languages it covers"""
-    with open(resource_filename("gftools", "udhr_all.txt"), encoding="utf-8") as doc:
-        uhdr = doc.read()
+    ref = importlib.resources.files("gftools") / "udhr_all.txt"
+    with importlib.resources.as_file(ref) as doc:
+        uhdr = doc.read_text()
 
     cmap = set(ttFont.getBestCmap())
     words = []
@@ -672,7 +676,9 @@ def shell_quote(s: Union[str, Path]) -> str:
 
 
 def github_user_repo(github_url):
-    pattern = r"https?://w?w?w?\.?github\.com/(?P<user>[^/]+)/(?P<repo>[^/^.]+)"
+    if github_url.endswith(".git"):
+        github_url = github_url[:-4]
+    pattern = r"https?://w?w?w?\.?github\.com/(?P<user>[^/]+)/(?P<repo>[^/]+)"
     match = re.search(pattern, github_url)
     if not match:
         raise ValueError(
@@ -685,3 +691,11 @@ def has_gh_token():
     if "GH_TOKEN" in os.environ:
         return True
     return False
+
+
+def parse_codepoint(codepoint: str) -> int:
+    # https://github.com/googlefonts/ufomerge/blob/2257a1d3807a4eec9b515aa98e059383f7814d9a/Lib/ufomerge/cli.py#L118-L126
+    if codepoint.startswith(("U+", "u+", "0x", "0X")):
+        return int(codepoint[2:], 16)
+    else:
+        return int(codepoint)
